@@ -2,7 +2,7 @@
 nydus.db.base
 ~~~~~~~~~~~~~
 
-:copyright: (c) 2011 DISQUS.
+:copyright: (c) 2011-2012 DISQUS.
 :license: Apache License 2.0, see LICENSE for more details.
 """
 
@@ -145,10 +145,18 @@ class RoundRobinRouter(BaseRouter):
     retryable = True
 
     # How many requests to serve in a situation when a host is down before
-    # the down hosts are retried
+    # the down hosts are assesed for readmittance back into the pool of serving
+    # requests.
+    #
+    # If the attempt_reconnect_threshold is hit, it does not guarantee that the
+    # down hosts will be put back - only that the router will CHECK to see if
+    # the hosts CAN be put back.  The elegibility of a host being put back is
+    # handlede in the check_down_connections method, which by default will
+    # readmit a host if it was marked down more than retry_timeout seconds ago.
     attempt_reconnect_threshold = 100000
 
-    # Retry a down connection after this timeout
+    # Number of seconds a host must be marked down before it is elligable to be
+    # put back in the pool and retried.
     retry_timeout = 30
 
     def __init__(self, *args, **kwargs):
@@ -164,9 +172,24 @@ class RoundRobinRouter(BaseRouter):
         except ValueError:
             raise cls.InvalidDBNum()
 
+    def check_down_connections(self):
+        """
+        Iterates through all connections which were previously listed as unavailable
+        and marks any that have expired their retry_timeout as being up.
+        """
+        now = time.time()
+
+        for db_num, marked_down_at in self._down_connections.items():
+            if marked_down_at + self.retry_timeout <= now:
+                self.mark_connection_up(db_num)
+
     def flush_down_connections(self):
+        """
+        Marks all connections which were previously listed as unavailable as being up.
+        """
         self._get_db_attempts = 0
-        self._down_connections = {}
+        for db_num in self._down_connections.keys():
+            self.mark_connection_up(db_num)
 
     def mark_connection_down(self, db_num):
         db_num = self.ensure_db_num(db_num)
@@ -187,7 +210,7 @@ class RoundRobinRouter(BaseRouter):
         self._get_db_attempts += 1
 
         if self._get_db_attempts > self.attempt_reconnect_threshold:
-            self.flush_down_connections()
+            self.check_down_connections()
 
         if retry_for is not None:
             self.mark_connection_down(retry_for)
